
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//

//
//  product: ro-eth-lc (ModuleID = RO_ETH_LC)
//  configuration: digital-output
//  programming language: vc
//
//
//  (c) DEDITEC GmbH, 2009-2021
//  web: http://www.deditec.de/
//  mail: vertrieb@deditec.de
//
//
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//
//
// Please include the following library on linking: delib_eth64.lib
//
// This can be done at the project settings (Project/Settings/Link -> 
// Object/library modules) .. extend the existing line with the ending 
// "delib_eth64.lib"		(with quotation marks)

#include <windows.h>
#include <stdio.h>
#include "conio.h"

#include "library\delib\delib.h"
#include "library\delib\delib_error_codes.h"


// ---------------------------------------------------------------------------
// GetLastError function

BOOL IsError()
{
	unsigned char msg[500];
	unsigned long error_code = DapiGetLastError();

	if (error_code != DAPI_ERR_NONE)
	{
		DapiGetLastErrorText((unsigned char*) msg, sizeof(msg));
		printf("Error Code = 0x%x * Message = %s\n", error_code, msg);

		DapiClearLastError();

		return TRUE;
	}

	return FALSE;
}

//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************
//***********************************************************************************

void main(void)
{
	unsigned long handle;
	unsigned long value;
	// ---------------------------------------------------------------------------
	// Open Module
	DAPI_OPENMODULEEX_STRUCT open_buffer;
	strcpy((char*) open_buffer.address, "192.168.1.1");
	open_buffer.portno = 9912;
	open_buffer.timeout = 5000;
	open_buffer.encryption_type = 0;

	printf("Opening module\n");
	printf("IP address: %s\n", open_buffer.address);
	printf("Port: %d\n", open_buffer.portno);
	printf("Timeout: %d\n", open_buffer.timeout);
	printf("Encryption Type: %d\n", open_buffer.encryption_type);
	printf("Module-ID: ETHERNET_MODULE(%d dez)\n", ETHERNET_MODULE);

	handle = DapiOpenModuleEx(ETHERNET_MODULE, 0, (unsigned char*) &open_buffer, 0);

	// ---------------------------------------------------------------------------
	// Module not found!

	if (handle==0)
	{
		printf("Could not open module!\n");
		printf("Press any key to exit\n");
		getch();
		return;
	}

	// ---------------------------------------------------------------------------
	// Module found!
	

	printf("Module has been opened\n");
	printf("Module handle = %x\n", handle);

	// ---------------------------------------------------------------------------
	// Show config of module
	
	value = DapiSpecialCommand(handle, DAPI_SPECIAL_CMD_GET_MODULE_CONFIG, DAPI_SPECIAL_GET_MODULE_CONFIG_PAR_DO, 0, 0);
	IsError();
	printf("Configuration of the module: no. of digital outputs %d\n", value);
	printf("Press any key to continue\n");
	getch();

	// ---------------------------------------------------------------------------
	// Write output channels

	DapiDOSet1(handle, 0, 1);
	IsError();
	printf("Output channel 0 has been switched on\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet1(handle, 0, 0);
	IsError();
	printf("Output channel 0 has been switched off\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet1(handle, 1, 1);
	IsError();
	printf("Output channel 1 has been switched on\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet1(handle, 1, 0);
	IsError();
	printf("Output channel 1 has been switched off\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet16(handle, 0, 0xff); //hexadecimal
	IsError();
	printf("Output channel 0-7 have been switched on and\n");
	printf("Output channel 8-15 have been switched off and\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet16(handle, 0, 0);
	IsError();
	printf("Output channel 0-15 have been switched off\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet8(handle, 0, 255);
	IsError();
	printf("Output channel 0-7 have been switched on\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet8(handle, 0, 0);
	IsError();
	printf("Output channel 0-7 have been switched off\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet8(handle, 8, 255);
	IsError();
	printf("Output channel 8-15 have been switched on\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet8(handle, 8, 0);
	IsError();
	printf("Output channel 8-15 have been switched off\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet32(handle, 0, 0xffffffff); //hexadecimal
	IsError();
	printf("Output channel 0-31 have been switched on\n");
	printf("Press any key to continue\n");
	getch();

	DapiDOSet32(handle, 0, 0);
	IsError();
	printf("Output channel 0-31 have been switched off\n");
	printf("Press any key to continue\n");
	getch();

	// ---------------------------------------------------------------------------
    // Write and readback output channels

	DapiDOSet8(handle, 0, 3);
	IsError();
	printf("Output channel 0-1 have been switched on\n");
	printf("Press any key to continue\n");
	getch();

	value = DapiDOReadback32(handle, 0);
	IsError();
	printf("Readback output channel 0-31\n");
	printf("value = %d\n", value);
	printf("Press any key to continue\n");
	getch();

	DapiDOSet8(handle, 0, 0);
	IsError();
	printf("Output channel 0-1 have been switched off\n");
	printf("Press any key to continue\n");
	getch();

	value = DapiDOReadback32(handle, 0);
	IsError();
	printf("Readback output channel 0-31\n");
	printf("value = %d\n", value);
	printf("Press any key to continue\n");
	getch();

	// ---------------------------------------------------------------------------
	// Set timeout of output channels to 5 seconds 

	DapiSpecialCommand(handle, DAPI_SPECIAL_CMD_TIMEOUT, DAPI_SPECIAL_TIMEOUT_SET_VALUE_SEC, 5, 0);
	IsError();
	printf("Timeout has been set to 5 seconds\n");
	printf("Press any key to continue\n");
	getch();

	// ---------------------------------------------------------------------------
	// Activate timeout and switch on output channels 0-31

	DapiSpecialCommand(handle, DAPI_SPECIAL_CMD_TIMEOUT, DAPI_SPECIAL_TIMEOUT_ACTIVATE, 0, 0);
	IsError();
	DapiDOSet32(handle, 0, 0xffffffff); //hexadecimal
	IsError();
	printf("Timeout has been activated\n");
	printf("Output channels 0-31 has been switched on and will be switched off automatically after 5 seconds\n");
	printf("Press any key to continue\n");
	getch();

	// ---------------------------------------------------------------------------
	// Deactivate timeout
	
	DapiSpecialCommand(handle, DAPI_SPECIAL_CMD_TIMEOUT, DAPI_SPECIAL_TIMEOUT_DEACTIVATE, 0, 0);
	IsError();
	printf("Timeout has been deactivated\n");
	printf("Press any key to continue\n");
	getch();

	// ---------------------------------------------------------------------------
	// Close Module

	DapiCloseModule(handle);
	printf("Module closed\n");
	printf("End of program!\n");
	printf("Press any key to exit\n");
	getch();

	return ;
}
